@@ -1,6 +1,6 @@
 // This module implements the QsciAPIs class.
 //
-// Copyright (c) 2017 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2019 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -128,7 +128,7 @@ private:
 
 // The worker thread ctor.
 QsciAPIsWorker::QsciAPIsWorker(QsciAPIs *apis)
-    : proxy(apis), prepared(0), abort(false)
+    : prepared(0), proxy(apis), abort(false)
 {
 }
 
@@ -250,6 +250,9 @@ bool QsciAPIs::event(QEvent *e)
         emit apiPreparationFinished();
 
         return true;
+
+    default:
+        break;
     }
 
     return QObject::event(e);
