@@ -2,9 +2,12 @@
 #define APPLICATION_H
 
 #include <QApplication>
+#include <QKeySequence>
+
+class QAction;
+class QTranslator;
 
 class MainWindow;
-class QTranslator;
 
 class Application : public QApplication
 {
@@ -12,16 +15,21 @@ class Application : public QApplication
 
 public:
     explicit Application(int& argc, char** argv);
-    virtual ~Application();
+    ~Application() override;
 
-    bool dontShowMainWindow() { return m_dontShowMainWindow; }
+    bool dontShowMainWindow() const { return m_dontShowMainWindow; }
 
     MainWindow* mainWindow() { return m_mainWindow; }
 
+    // DB4S version number as string
     static QString versionString();
+    // Version of DB4S and dependencies as string
+    static QString versionInformation();
+
+    static void reloadSettings();
 
 protected:
-    bool event(QEvent* event);
+    bool event(QEvent* event) override;
 
 private:
     bool m_dontShowMainWindow;
@@ -30,4 +38,6 @@ private:
     QTranslator* m_translatorApp;
 };
 
+void addShortcutsTooltip(QAction* action, const QList<QKeySequence>& extraKeys = QList<QKeySequence>());
+
 #endif
