@@ -1,9 +1,10 @@
 #ifndef DOTENVFORMAT_H
 #define DOTENVFORMAT_H
 
-#include <QIODevice>
 #include <QSettings>
 
+class QIODevice;
+
 class DotenvFormat
 {
 public:
