@@ -1,14 +1,19 @@
 #ifndef EDITTABLEDIALOG_H
 #define EDITTABLEDIALOG_H
 
-#include "sqlitetypes.h"
+#include "sql/ObjectIdentifier.h"
+#include "sql/sqlitetypes.h"
+
+#include <map>
 
 #include <QDialog>
 
 class DBBrowserDB;
-class QTreeWidgetItem;
 class ForeignKeyEditorDelegate;
 
+class QTableWidgetItem;
+class QTreeWidgetItem;
+
 namespace Ui {
 class EditTableDialog;
 }
@@ -19,10 +24,10 @@ class EditTableDialog : public QDialog
 
 public:
     explicit EditTableDialog(DBBrowserDB& pdb, const sqlb::ObjectIdentifier& tableName, bool createTable, QWidget* parent = nullptr);
-    ~EditTableDialog();
+    ~EditTableDialog() override;
 
 protected:
-    void keyPressEvent(QKeyEvent *evt);
+    void keyPressEvent(QKeyEvent *evt) override;
 
 private:
     enum Columns {
@@ -34,42 +39,63 @@ private:
         kUnique = 5,
         kDefault = 6,
         kCheck = 7,
-        kForeignKey = 8
+        kCollation = 8,
+        kForeignKey = 9
+    };
+
+    enum ConstraintColumns {
+        kConstraintColumns = 0,
+        kConstraintType = 1,
+        kConstraintName = 2,
+        kConstraintSql = 3
+    };
+
+    enum MoveFieldDirection
+    {
+        MoveUp,
+        MoveDown,
+        MoveTop,
+        MoveBottom
     };
 
     void updateColumnWidth();
     void updateSqlText();
 
-    void moveCurrentField(bool down);
+    void moveCurrentField(MoveFieldDirection dir);
 
 private slots:
     void populateFields();
+    void populateConstraints();
     void addField();
     void removeField();
     void fieldSelectionChanged();
-    virtual void accept();
-    virtual void reject();
+    void accept() override;
+    void reject() override;
     void checkInput();
-    void itemChanged(QTreeWidgetItem* item, int column);
-    void updateTypes(QObject *object);
-    bool eventFilter(QObject *object, QEvent *event);
-    void updateTypes();
+    void fieldItemChanged(QTreeWidgetItem* item, int column);
+    void constraintItemChanged(QTableWidgetItem* item);
+    void updateTypeAndCollation(QObject *object);
+    bool eventFilter(QObject *object, QEvent *event) override;
+    void updateTypeAndCollation();
     void moveUp();
     void moveDown();
+    void moveTop();
+    void moveBottom();
     void setWithoutRowid(bool without_rowid);
     void changeSchema(const QString& schema);
+    void removeConstraint();
+    void addConstraint(sqlb::Constraint::ConstraintTypes type);
 
 private:
     Ui::EditTableDialog* ui;
     DBBrowserDB& pdb;
     ForeignKeyEditorDelegate* m_fkEditorDelegate;
     sqlb::ObjectIdentifier curTable;
+    std::map<QString, QString> trackColumns;
     sqlb::Table m_table;
-    QStringList types;
-    QStringList fields;
     bool m_bNewTable;
-    QString m_sRestorePointName;
-    bool m_bForeignKeysEnabled;
+    std::string m_sRestorePointName;
+    QStringList m_collationList;
 };
 
 #endif
