@@ -2,7 +2,7 @@
 #define FILTERLINEEDIT_H
 
 #include <QLineEdit>
-#include <QList>
+#include <vector>
 
 class QTimer;
 class QKeyEvent;
@@ -12,27 +12,33 @@ class FilterLineEdit : public QLineEdit
     Q_OBJECT
 
 public:
-    explicit FilterLineEdit(QWidget* parent, QList<FilterLineEdit*>* filters, int columnnum);
+    explicit FilterLineEdit(QWidget* parent, std::vector<FilterLineEdit*>* filters = nullptr, size_t columnnum = 0);
 
     // Override methods for programatically changing the value of the line edit
     void clear();
     void setText(const QString& text);
 
+    void setConditionFormatContextMenuEnabled(bool enable) { conditional_format = enable; }
+
 private slots:
     void delayedSignalTimerTriggered();
 
 signals:
     void delayedTextChanged(QString text);
+    void addFilterAsCondFormat(QString text);
+    void clearAllCondFormats();
+    void editCondFormats();
 
 protected:
-    void keyReleaseEvent(QKeyEvent* event);
-    void setFilterHelper(const QString& filterOperator);
+    void keyReleaseEvent(QKeyEvent* event) override;
+    void setFilterHelper(const QString& filterOperator, const QString& operatorSuffix = QString());
 
 private:
-    QList<FilterLineEdit*>* filterList;
-    int columnNumber;
+    std::vector<FilterLineEdit*>* filterList;
+    size_t columnNumber;
     QTimer* delaySignalTimer;
     QString lastValue;
+    bool conditional_format;
 
 private slots:
     void showContextMenu(const QPoint &pos);
