@@ -2,14 +2,13 @@
 #define IMPORTCSVDIALOG_H
 
 #include "csvparser.h"
-#include "sqlitetypes.h"
+#include "sql/sqlitetypes.h"
 
 #include <QDialog>
 #include <functional>
 
 class DBBrowserDB;
 class QCompleter;
-class QListWidgetItem;
 
 namespace Ui {
 class ImportCsvDialog;
@@ -20,11 +19,11 @@ class ImportCsvDialog : public QDialog
     Q_OBJECT
 
 public:
-    explicit ImportCsvDialog(const QStringList& filenames, DBBrowserDB* db, QWidget* parent = nullptr);
-    ~ImportCsvDialog();
+    explicit ImportCsvDialog(const std::vector<QString>& filenames, DBBrowserDB* db, QWidget* parent = nullptr);
+    ~ImportCsvDialog() override;
 
 private slots:
-    virtual void accept();
+    void accept() override;
     void updatePreview();
     void checkInput();
     void selectFiles();
@@ -34,26 +33,35 @@ private slots:
     void toggleAdvancedSection(bool show);
 
 private:
+
+    // Positions for combos starting at the bottom
+    enum {OtherCode = 1,
+          OtherPrintable = 2};
+
     Ui::ImportCsvDialog* ui;
-    QStringList csvFilenames;
+    std::vector<QString> csvFilenames;
     QString selectedFile;
     DBBrowserDB* pdb;
     QCompleter* encodingCompleter;
     QStringList dontAskForExistingTableAgain;
 
-    CSVParser::ParserResult parseCSV(const QString& fileName, std::function<bool(size_t, CSVRow)> rowFunction, size_t count = 0);
-    sqlb::FieldVector generateFieldList(const QString& filename);
+    CSVParser::ParserResult parseCSV(const QString& fileName, std::function<bool(size_t, CSVRow)> rowFunction, size_t count = 0) const;
+    sqlb::FieldVector generateFieldList(const QString& filename) const;
 
     bool importCsv(const QString& f, const QString& n = QString());
 
-    void setQuoteChar(const QChar& c);
-    char currentQuoteChar() const;
+    void setQuoteChar(QChar c);
+    QChar currentQuoteChar() const;
 
-    void setSeparatorChar(const QChar& c);
-    char currentSeparatorChar() const;
+    void setSeparatorChar(QChar c);
+    QChar currentSeparatorChar() const;
 
     void setEncoding(const QString& sEnc);
     QString currentEncoding() const;
+
+    std::string currentOnConflictStrategy() const;
+
+    char32_t toUtf8(const QString& s) const;
 };
 
 #endif
