@@ -1,13 +1,17 @@
 #ifndef PLOTDOCK_H
 #define PLOTDOCK_H
 
+#include "Palette.h"
+
 #include <QDialog>
 #include <QVariant>
-#include <QMenu>
+
+class QMenu;
+class QPrinter;
+class QTreeWidgetItem;
+class QCPAxis;
 
 class SqliteTableModel;
-class QTreeWidgetItem;
-class QPrinter;
 struct BrowseDataTableSettings;
 
 namespace Ui {
@@ -20,7 +24,7 @@ class PlotDock : public QDialog
 
 public:
     explicit PlotDock(QWidget* parent = nullptr);
-    ~PlotDock();
+    ~PlotDock() override;
 
     struct PlotSettings
     {
@@ -68,7 +72,7 @@ public slots:
     void updatePlot(SqliteTableModel* model, BrowseDataTableSettings* settings = nullptr, bool update = true, bool keepOrResetSelection = true);
     void fetchAllData();
     void resetPlot();
-    virtual void reject();
+    void reject() override;
 
 signals:
     void pointsSelected(int firstIndex, int count);
@@ -78,8 +82,9 @@ private:
     {
         PlotColumnField = 0,
         PlotColumnX = 1,
-        PlotColumnY = 2,
-        PlotColumnType = 3,
+        PlotColumnY1 = 2,
+        PlotColumnY2 = 3,
+        PlotColumnType = 4,
     };
 
     Ui::PlotDock* ui;
@@ -89,6 +94,9 @@ private:
     QMenu* m_contextMenu;
     bool m_showLegend;
     bool m_stackedBars;
+    Palette m_graphPalette;
+    QList<QCPAxis *> yAxes;
+    QList<int> PlotColumnY;
 
     /*!
      * \brief guessdatatype try to parse the first 10 rows and decide the datatype
@@ -96,7 +104,7 @@ private:
      * \param column index of the column to check
      * \return the guessed datatype
      */
-    QVariant::Type guessDataType(SqliteTableModel* model, int column);
+    QVariant::Type guessDataType(SqliteTableModel* model, int column) const;
     void adjustBars();
 
 private slots:
