@@ -4,24 +4,25 @@
 #include "Settings.h"
 #include "Application.h"
 #include "MainWindow.h"
-#include "RemoteDatabase.h"
+#include "RemoteNetwork.h"
 #include "FileExtensionManager.h"
+#include "ProxyDialog.h"
 
 #include <QDir>
 #include <QColorDialog>
 #include <QMessageBox>
 #include <QKeyEvent>
 #include <QStandardPaths>
+#include <QStyledItemDelegate>
 
-PreferencesDialog::PreferencesDialog(QWidget* parent)
+PreferencesDialog::PreferencesDialog(QWidget* parent, Tabs tab)
     : QDialog(parent),
       ui(new Ui::PreferencesDialog),
+      m_proxyDialog(new ProxyDialog(this)),
       m_dbFileExtensions(Settings::getValue("General", "DBFileExtensions").toString().split(";;"))
 {
     ui->setupUi(this);
     ui->treeSyntaxHighlighting->setColumnHidden(0, true);
-    ui->labelDatabaseDefaultSqlText->setVisible(false);
-    ui->editDatabaseDefaultSqlText->setVisible(false);
     ui->tableClientCerts->setColumnHidden(0, true);
 
     ui->fr_bin_bg->installEventFilter(this);
@@ -31,8 +32,8 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
     ui->fr_null_bg->installEventFilter(this);
     ui->fr_null_fg->installEventFilter(this);
 
-    connect(ui->comboDataBrowserFont, SIGNAL(currentIndexChanged(int)), this, SLOT(updatePreviewFont()));
-    connect(ui->spinDataBrowserFontSize, SIGNAL(valueChanged(int)), this, SLOT(updatePreviewFont()));
+    connect(ui->comboDataBrowserFont, static_cast<void (QFontComboBox::*)(int)>(&QFontComboBox::currentIndexChanged), this, &PreferencesDialog::updatePreviewFont);
+    connect(ui->spinDataBrowserFontSize, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &PreferencesDialog::updatePreviewFont);
 
 #ifndef CHECKNEWVERSION
     ui->labelUpdates->setVisible(false);
@@ -41,8 +42,13 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 
     loadSettings();
 
+    connect(ui->appStyleCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(adjustColorsToStyle(int)));
+
     // Avoid different heights due to having check boxes or not
     ui->treeSyntaxHighlighting->setUniformRowHeights(true);
+
+    // Set current tab
+    ui->tabWidget->setCurrentIndex(tab);
 }
 
 /*
@@ -56,6 +62,7 @@ PreferencesDialog::~PreferencesDialog()
 void PreferencesDialog::chooseLocation()
 {
     QString s = FileDialog::getExistingDirectory(
+                NoSpecificType,
                 this,
                 tr("Choose a directory"),
                 QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
@@ -68,7 +75,7 @@ void PreferencesDialog::loadSettings()
 {
     ui->encodingComboBox->setCurrentIndex(ui->encodingComboBox->findText(Settings::getValue("db", "defaultencoding").toString(), Qt::MatchFixedString));
     ui->comboDefaultLocation->setCurrentIndex(Settings::getValue("db", "savedefaultlocation").toInt());
-    ui->locationEdit->setText(Settings::getValue("db", "defaultlocation").toString());
+    ui->locationEdit->setText(QDir::toNativeSeparators(Settings::getValue("db", "defaultlocation").toString()));
     ui->checkUpdates->setChecked(Settings::getValue("checkversion", "enabled").toBool());
 
     ui->checkHideSchemaLinebreaks->setChecked(Settings::getValue("db", "hideschemalinebreaks").toBool());
@@ -79,12 +86,13 @@ void PreferencesDialog::loadSettings()
     ui->defaultFieldTypeComboBox->addItems(DBBrowserDB::Datatypes);
 
     int defaultFieldTypeIndex = Settings::getValue("db", "defaultfieldtype").toInt();
-
     if (defaultFieldTypeIndex < DBBrowserDB::Datatypes.count())
     {
         ui->defaultFieldTypeComboBox->setCurrentIndex(defaultFieldTypeIndex);
     }
 
+    ui->spinStructureFontSize->setValue(Settings::getValue("db", "fontsize").toInt());
+
     // Gracefully handle the preferred Data Browser font not being available
     int matchingFont = ui->comboDataBrowserFont->findText(Settings::getValue("databrowser", "font").toString(), Qt::MatchExactly);
     if (matchingFont == -1)
@@ -101,6 +109,7 @@ void PreferencesDialog::loadSettings()
 
     ui->spinSymbolLimit->setValue(Settings::getValue("databrowser", "symbol_limit").toInt());
     ui->spinCompleteThreshold->setValue(Settings::getValue("databrowser", "complete_threshold").toInt());
+    ui->checkShowImagesInline->setChecked(Settings::getValue("databrowser", "image_preview").toBool());
     ui->txtNull->setText(Settings::getValue("databrowser", "null_text").toString());
     ui->txtBlob->setText(Settings::getValue("databrowser", "blob_text").toString());
     ui->editFilterEscape->setText(Settings::getValue("databrowser", "filter_escape").toString());
@@ -108,7 +117,7 @@ void PreferencesDialog::loadSettings()
 
     for(int i=0; i < ui->treeSyntaxHighlighting->topLevelItemCount(); ++i)
     {
-        QString name = ui->treeSyntaxHighlighting->topLevelItem(i)->text(0);
+        std::string name = ui->treeSyntaxHighlighting->topLevelItem(i)->text(0).toStdString();
         QString colorname = Settings::getValue("syntaxhighlighter", name + "_colour").toString();
         QColor color = QColor(colorname);
         ui->treeSyntaxHighlighting->topLevelItem(i)->setTextColor(2, color);
@@ -124,7 +133,7 @@ void PreferencesDialog::loadSettings()
     // Remote settings
     ui->checkUseRemotes->setChecked(Settings::getValue("remote", "active").toBool());
     {
-        auto ca_certs = static_cast<Application*>(qApp)->mainWindow()->getRemote().caCertificates();
+        auto ca_certs = RemoteNetwork::get().caCertificates();
         ui->tableCaCerts->setRowCount(ca_certs.size());
         for(int i=0;i<ca_certs.size();i++)
         {
@@ -160,7 +169,7 @@ void PreferencesDialog::loadSettings()
                 addClientCertToTable(file, cert);
         }
     }
-    ui->editRemoteCloneDirectory->setText(Settings::getValue("remote", "clonedirectory").toString());
+    ui->editRemoteCloneDirectory->setText(QDir::toNativeSeparators(Settings::getValue("remote", "clonedirectory").toString()));
 
     // Gracefully handle the preferred Editor font not being available
     matchingFont = ui->comboEditorFont->findText(Settings::getValue("editor", "font").toString(), Qt::MatchExactly);
@@ -178,12 +187,19 @@ void PreferencesDialog::loadSettings()
     ui->checkCompleteUpper->setChecked(Settings::getValue("editor", "upper_keywords").toBool());
     ui->checkErrorIndicators->setChecked(Settings::getValue("editor", "error_indicators").toBool());
     ui->checkHorizontalTiling->setChecked(Settings::getValue("editor", "horizontal_tiling").toBool());
+    ui->checkCloseButtonOnTabs->setChecked(Settings::getValue("editor", "close_button_on_tabs").toBool());
 
     ui->listExtensions->addItems(Settings::getValue("extensions", "list").toStringList());
     ui->checkRegexDisabled->setChecked(Settings::getValue("extensions", "disableregex").toBool());
     ui->checkAllowLoadExtension->setChecked(Settings::getValue("extensions", "enable_load_extension").toBool());
     fillLanguageBox();
-    ui->toolbarStyleComboBox->setCurrentIndex(Settings::getValue("General", "toolbarStyle").toInt());
+    ui->appStyleCombo->setCurrentIndex(Settings::getValue("General", "appStyle").toInt());
+    ui->toolbarStyleComboMain->setCurrentIndex(Settings::getValue("General", "toolbarStyle").toInt());
+    ui->toolbarStyleComboStructure->setCurrentIndex(Settings::getValue("General", "toolbarStyleStructure").toInt());
+    ui->toolbarStyleComboBrowse->setCurrentIndex(Settings::getValue("General", "toolbarStyleBrowse").toInt());
+    ui->toolbarStyleComboSql->setCurrentIndex(Settings::getValue("General", "toolbarStyleSql").toInt());
+    ui->toolbarStyleComboEditCell->setCurrentIndex(Settings::getValue("General", "toolbarStyleEditCell").toInt());
+    ui->spinGeneralFontSize->setValue(Settings::getValue("General", "fontsize").toInt());
 }
 
 void PreferencesDialog::saveSettings()
@@ -197,13 +213,14 @@ void PreferencesDialog::saveSettings()
     Settings::setValue("db", "foreignkeys", ui->foreignKeysCheckBox->isChecked());
     Settings::setValue("db", "prefetchsize", ui->spinPrefetchSize->value());
     Settings::setValue("db", "defaultsqltext", ui->editDatabaseDefaultSqlText->text());
-
     Settings::setValue("db", "defaultfieldtype", ui->defaultFieldTypeComboBox->currentIndex());
+    Settings::setValue("db", "fontsize", ui->spinStructureFontSize->value());
 
     Settings::setValue("checkversion", "enabled", ui->checkUpdates->isChecked());
 
     Settings::setValue("databrowser", "font", ui->comboDataBrowserFont->currentText());
     Settings::setValue("databrowser", "fontsize", ui->spinDataBrowserFontSize->value());
+    Settings::setValue("databrowser", "image_preview", ui->checkShowImagesInline->isChecked());
     saveColorSetting(ui->fr_null_fg, "null_fg");
     saveColorSetting(ui->fr_null_bg, "null_bg");
     saveColorSetting(ui->fr_reg_fg, "reg_fg");
@@ -219,7 +236,7 @@ void PreferencesDialog::saveSettings()
 
     for(int i=0; i < ui->treeSyntaxHighlighting->topLevelItemCount(); ++i)
     {
-        QString name = ui->treeSyntaxHighlighting->topLevelItem(i)->text(0);
+        std::string name = ui->treeSyntaxHighlighting->topLevelItem(i)->text(0).toStdString();
         Settings::setValue("syntaxhighlighter", name + "_colour", ui->treeSyntaxHighlighting->topLevelItem(i)->text(2));
         Settings::setValue("syntaxhighlighter", name + "_bold", ui->treeSyntaxHighlighting->topLevelItem(i)->checkState(3) == Qt::Checked);
         Settings::setValue("syntaxhighlighter", name + "_italic", ui->treeSyntaxHighlighting->topLevelItem(i)->checkState(4) == Qt::Checked);
@@ -235,6 +252,7 @@ void PreferencesDialog::saveSettings()
     Settings::setValue("editor", "upper_keywords", ui->checkCompleteUpper->isChecked());
     Settings::setValue("editor", "error_indicators", ui->checkErrorIndicators->isChecked());
     Settings::setValue("editor", "horizontal_tiling", ui->checkHorizontalTiling->isChecked());
+    Settings::setValue("editor", "close_button_on_tabs", ui->checkCloseButtonOnTabs->isChecked());
 
     QStringList extList;
     for(const QListWidgetItem* item : ui->listExtensions->findItems(QString("*"), Qt::MatchWrap | Qt::MatchWildcard))
@@ -305,8 +323,16 @@ void PreferencesDialog::saveSettings()
                                  tr("The language will change after you restart the application."));
 
     Settings::setValue("General", "language", newLanguage);
-    Settings::setValue("General", "toolbarStyle", ui->toolbarStyleComboBox->currentIndex());
+    Settings::setValue("General", "appStyle", ui->appStyleCombo->currentIndex());
+    Settings::setValue("General", "toolbarStyle", ui->toolbarStyleComboMain->currentIndex());
+    Settings::setValue("General", "toolbarStyleStructure", ui->toolbarStyleComboStructure->currentIndex());
+    Settings::setValue("General", "toolbarStyleBrowse", ui->toolbarStyleComboBrowse->currentIndex());
+    Settings::setValue("General", "toolbarStyleSql", ui->toolbarStyleComboSql->currentIndex());
+    Settings::setValue("General", "toolbarStyleEditCell", ui->toolbarStyleComboEditCell->currentIndex());
     Settings::setValue("General", "DBFileExtensions", m_dbFileExtensions.join(";;") );
+    Settings::setValue("General", "fontsize", ui->spinGeneralFontSize->value());
+
+    m_proxyDialog->saveSettings();
 
     accept();
 
@@ -368,6 +394,7 @@ bool PreferencesDialog::eventFilter(QObject *obj, QEvent *event)
 void PreferencesDialog::addExtension()
 {
     QString file = FileDialog::getOpenFileName(
+                OpenExtensionFile,
                 this,
                 tr("Select extension file"),
                 tr("Extensions(*.so *.dylib *.dll);;All files(*)"));
@@ -445,9 +472,15 @@ void PreferencesDialog::fillLanguageBox()
     ui->languageComboBox->removeItem(index);
     ui->languageComboBox->insertItem(0, chosenIcon, chosenLanguage, chosenLocale);
     ui->languageComboBox->setCurrentIndex(0);
+
+    // This is a workaround needed for QDarkStyleSheet.
+    // See https://github.com/ColinDuquesnoy/QDarkStyleSheet/issues/169
+    QStyledItemDelegate* styledItemDelegate = new QStyledItemDelegate(ui->languageComboBox);
+    ui->languageComboBox->setItemDelegate(styledItemDelegate);
+
 }
 
-void PreferencesDialog::loadColorSetting(QFrame *frame, const QString & settingName)
+void PreferencesDialog::loadColorSetting(QFrame *frame, const std::string& settingName)
 {
     QColor color = QColor(Settings::getValue("databrowser", settingName + "_colour").toString());
     setColorSetting(frame, color);
@@ -483,17 +516,42 @@ void PreferencesDialog::setColorSetting(QFrame *frame, const QColor &color)
     palette.setColor(frame->backgroundRole(), color);
     frame->setPalette(palette);
 
+    frame->setStyleSheet(QString(".QFrame {background-color: %2}").arg(color.name()));
+
     palette = line->palette();
     palette.setColor(role, color);
     line->setPalette(palette);
+
+    line->setStyleSheet(QString(".QLineEdit {color: %1; background-color: %2}").arg(palette.color(line->foregroundRole()).name(),
+                                                                       palette.color(line->backgroundRole()).name()));
 }
 
-void PreferencesDialog::saveColorSetting(QFrame *frame, const QString & settingName)
+void PreferencesDialog::saveColorSetting(QFrame* frame, const std::string& settingName)
 {
     Settings::setValue("databrowser", settingName + "_colour",
         frame->palette().color(frame->backgroundRole()));
 }
 
+void PreferencesDialog::adjustColorsToStyle(int style)
+{
+    Settings::AppStyle appStyle = static_cast<Settings::AppStyle>(style);
+    setColorSetting(ui->fr_null_fg, Settings::getDefaultColorValue("databrowser", "null_fg_colour", appStyle));
+    setColorSetting(ui->fr_null_bg, Settings::getDefaultColorValue("databrowser", "null_bg_colour", appStyle));
+    setColorSetting(ui->fr_bin_fg, Settings::getDefaultColorValue("databrowser", "bin_fg_colour", appStyle));
+    setColorSetting(ui->fr_bin_bg, Settings::getDefaultColorValue("databrowser", "bin_bg_colour", appStyle));
+    setColorSetting(ui->fr_reg_fg, Settings::getDefaultColorValue("databrowser", "reg_fg_colour", appStyle));
+    setColorSetting(ui->fr_reg_bg, Settings::getDefaultColorValue("databrowser", "reg_bg_colour", appStyle));
+
+    for(int i=0; i < ui->treeSyntaxHighlighting->topLevelItemCount(); ++i)
+    {
+        std::string name = ui->treeSyntaxHighlighting->topLevelItem(i)->text(0).toStdString();
+        QColor color = Settings::getDefaultColorValue("syntaxhighlighter", name + "_colour", appStyle);
+        ui->treeSyntaxHighlighting->topLevelItem(i)->setTextColor(2, color);
+        ui->treeSyntaxHighlighting->topLevelItem(i)->setBackgroundColor(2, color);
+        ui->treeSyntaxHighlighting->topLevelItem(i)->setText(2, color.name());
+    }
+}
+
 void PreferencesDialog::activateRemoteTab(bool active)
 {
     ui->tabWidget->setTabEnabled(ui->tabWidget->indexOf(ui->tabRemote), active);
@@ -503,7 +561,7 @@ void PreferencesDialog::addClientCertificate()
 {
     // Get certificate file to import and abort here if no file gets selected
     // NOTE: We assume here that this file contains both, certificate and private key!
-    QString path = FileDialog::getOpenFileName(this, tr("Import certificate file"), "*.pem");
+    QString path = FileDialog::getOpenFileName(OpenCertificateFile, this, tr("Import certificate file"), "*.pem");
     if(path.isEmpty())
         return;
 
@@ -575,11 +633,12 @@ void PreferencesDialog::addClientCertToTable(const QString& path, const QSslCert
 void PreferencesDialog::chooseRemoteCloneDirectory()
 {
     QString s = FileDialog::getExistingDirectory(
+                NoSpecificType,
                 this,
                 tr("Choose a directory"),
                 QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
 
-    if(!s.isEmpty())
+    if(!s.isEmpty() && QDir().mkpath(s))
         ui->editRemoteCloneDirectory->setText(s);
 }
 
@@ -620,3 +679,8 @@ void PreferencesDialog::on_buttonBox_clicked(QAbstractButton* button)
         }
     }
 }
+
+void PreferencesDialog::configureProxy()
+{
+    m_proxyDialog->show();
+}
