@@ -1,10 +1,10 @@
-#include <QJsonDocument>
-#include <QJsonArray>
-#include <QJsonObject>
 #include <QImage>
 
+#include "Data.h"
 #include "RemoteModel.h"
-#include "RemoteDatabase.h"
+#include "RemoteNetwork.h"
+
+using json = nlohmann::json;
 
 RemoteModelItem::RemoteModelItem(RemoteModelItem* parent) :
     m_parent(parent),
@@ -29,12 +29,12 @@ void RemoteModelItem::setValue(RemoteModelColumns column, QVariant value)
 
 void RemoteModelItem::appendChild(RemoteModelItem *item)
 {
-    m_children.append(item);
+    m_children.push_back(item);
 }
 
 RemoteModelItem* RemoteModelItem::child(int row) const
 {
-    return m_children.value(row);
+    return m_children[static_cast<size_t>(row)];
 }
 
 RemoteModelItem* RemoteModelItem::parent() const
@@ -44,13 +44,19 @@ RemoteModelItem* RemoteModelItem::parent() const
 
 int RemoteModelItem::childCount() const
 {
-    return m_children.count();
+    return static_cast<int>(m_children.size());
 }
 
 int RemoteModelItem::row() const
 {
     if(m_parent)
-        return m_parent->m_children.indexOf(const_cast<RemoteModelItem*>(this));
+    {
+        auto f = std::find( m_parent->m_children.begin(), m_parent->m_children.end(), const_cast<RemoteModelItem*>(this));
+        if(f == m_parent->m_children.end())
+            return -1;
+        else
+            return static_cast<int>(std::distance(m_parent->m_children.begin(), f));
+    }
 
     return 0;
 }
@@ -65,25 +71,32 @@ void RemoteModelItem::setFetchedDirectoryList(bool fetched)
     m_fetchedDirectoryList = fetched;
 }
 
-QList<RemoteModelItem*> RemoteModelItem::loadArray(const QJsonValue& value, RemoteModelItem* parent)
+std::vector<RemoteModelItem*> RemoteModelItem::loadArray(const json& array, RemoteModelItem* parent)
 {
-    QList<RemoteModelItem*> items;
+    std::vector<RemoteModelItem*> items;
 
     // Loop through all directory items
-    QJsonArray array = value.toArray();
-    for(int i=0;i<array.size();i++)
+    for(const auto& elem : array)
     {
         // Create a new model item with the specified parent
         RemoteModelItem* item = new RemoteModelItem(parent);
 
-        // Save all relevant values. If one of the values isn't set in the JSON document, an empty string
-        // will be stored
-        item->setValue(RemoteModelColumnName, array.at(i).toObject().value("name"));
-        item->setValue(RemoteModelColumnType, array.at(i).toObject().value("type"));
-        item->setValue(RemoteModelColumnUrl, array.at(i).toObject().value("url"));
-        item->setValue(RemoteModelColumnCommitId, array.at(i).toObject().value("commit_id"));
-        item->setValue(RemoteModelColumnSize, array.at(i).toObject().value("size"));
-        item->setValue(RemoteModelColumnLastModified, array.at(i).toObject().value("last_modified"));
+        // Save all relevant values. Some of the values are only available for databases.
+        item->setValue(RemoteModelColumnName, QString::fromStdString(elem["name"]));
+        item->setValue(RemoteModelColumnType, QString::fromStdString(elem["type"]));
+        item->setValue(RemoteModelColumnUrl, QString::fromStdString(elem["url"]));
+        item->setValue(RemoteModelColumnLastModified, isoDateTimeStringToLocalDateTimeString(QString::fromStdString(elem["last_modified"])));
+        if(item->value(RemoteModelColumnType).toString() == "database")
+        {
+            item->setValue(RemoteModelColumnCommitId, QString::fromStdString(elem["commit_id"]));
+            item->setValue(RemoteModelColumnSize, QString::number(static_cast<unsigned long>(elem["size"])));
+            item->setValue(RemoteModelColumnDefaultBranch, QString::fromStdString(elem["default_branch"]));
+            item->setValue(RemoteModelColumnLicence, QString::fromStdString(elem["licence"]));
+            item->setValue(RemoteModelColumnOneLineDescription, QString::fromStdString(elem["one_line_description"]));
+            item->setValue(RemoteModelColumnPublic, static_cast<bool>(elem["public"]));
+            item->setValue(RemoteModelColumnSha256, QString::fromStdString(elem["sha256"]));
+            item->setValue(RemoteModelColumnRepoModified, isoDateTimeStringToLocalDateTimeString(QString::fromStdString(elem["repo_modified"])));
+        }
 
         items.push_back(item);
     }
@@ -91,16 +104,11 @@ QList<RemoteModelItem*> RemoteModelItem::loadArray(const QJsonValue& value, Remo
     return items;
 }
 
-RemoteModel::RemoteModel(QObject* parent, RemoteDatabase& remote) :
+RemoteModel::RemoteModel(QObject* parent) :
     QAbstractItemModel(parent),
-    rootItem(new RemoteModelItem()),
-    remoteDatabase(remote)
+    headerList({tr("Name"), tr("Last modified"), tr("Size"), tr("Commit")}),
+    rootItem(new RemoteModelItem())
 {
-    // Initialise list of column names
-    headerList << tr("Name") << tr("Commit") << tr("Last modified") << tr("Size");
-
-    // Set up signals
-    connect(&remoteDatabase, &RemoteDatabase::gotDirList, this, &RemoteModel::parseDirectoryListing);
 }
 
 RemoteModel::~RemoteModel()
@@ -111,26 +119,32 @@ RemoteModel::~RemoteModel()
 void RemoteModel::setNewRootDir(const QString& url, const QString& cert)
 {
     // Get user name from client cert
-    currentUserName = remoteDatabase.getInfoFromClientCert(cert, RemoteDatabase::CertInfoUser);
+    currentUserName = RemoteNetwork::get().getInfoFromClientCert(cert, RemoteNetwork::CertInfoUser);
 
     // Save settings
     currentRootDirectory = url;
     currentClientCert = cert;
 
-    // Fetch root directory and put the reply data under the root item
-    remoteDatabase.fetch(currentRootDirectory, RemoteDatabase::RequestTypeDirectory, currentClientCert, QModelIndex());
+    // Fetch root directory
+    refresh();
 }
 
-void RemoteModel::parseDirectoryListing(const QString& json, const QVariant& userdata)
+void RemoteModel::refresh()
+{
+    // Fetch root directory and put the reply data under the root item
+    RemoteNetwork::get().fetch(currentRootDirectory, RemoteNetwork::RequestTypeCustom, currentClientCert, [this](const QByteArray& reply) {
+        parseDirectoryListing(reply, QModelIndex());
+    });
+}
+
+void RemoteModel::parseDirectoryListing(const QString& text, QModelIndex parent)
 {
     // Load new JSON root document assuming it's an array
-    QJsonDocument doc = QJsonDocument::fromJson(json.toUtf8());
-    if(doc.isNull() || !doc.isArray())
+    json array = json::parse(text.toStdString(), nullptr, false);
+    if(array.is_discarded() || !array.is_array())
         return;
-    QJsonArray array = doc.array();
 
     // Get model index to store the new data under
-    QModelIndex parent = userdata.toModelIndex();
     RemoteModelItem* parentItem = const_cast<RemoteModelItem*>(modelIndexToItem(parent));
 
     // An invalid model index indicates that this is a new root item. This means the old one needs to be entirely deleted first.
@@ -148,8 +162,8 @@ void RemoteModel::parseDirectoryListing(const QString& json, const QVariant& use
     }
 
     // Insert data
-    beginInsertRows(parent, 0, array.size());
-    QList<RemoteModelItem*> items = RemoteModelItem::loadArray(QJsonValue(array), parentItem);
+    std::vector<RemoteModelItem*> items = RemoteModelItem::loadArray(array, parentItem);
+    beginInsertRows(parent, 0, static_cast<int>(items.size() - 1));
     for(RemoteModelItem* item : items)
         parentItem->appendChild(item);
     endInsertRows();
@@ -206,6 +220,20 @@ QVariant RemoteModel::data(const QModelIndex& index, int role) const
             return QImage(":/icons/folder");
         else if(type == "database")
             return QImage(":/icons/database");
+    } else if(role == Qt::ToolTipRole) {
+        if(type == "database")
+        {
+            // Use URL to generate user name and database name. This avoids using the name of the parent item which
+            // might not contain the user name when the server sends a different directory structure.
+            QString result = "<b>" + item->value(RemoteModelColumnUrl).toUrl().path().mid(1) + "</b>";
+            if(!item->value(RemoteModelColumnOneLineDescription).toString().isEmpty())
+                result += "<br />" + item->value(RemoteModelColumnOneLineDescription).toString();
+            result += "<br />" + tr("Size: ") + humanReadableSize(item->value(RemoteModelColumnSize).toULongLong());
+            result += "<br />" + tr("Last Modified: ") + item->value(RemoteModelColumnLastModified).toString();
+            result += "<br />" + tr("Licence: ") + item->value(RemoteModelColumnLicence).toString();
+            result += "<br />" + tr("Default Branch: ") + item->value(RemoteModelColumnDefaultBranch).toString();
+            return result;
+        }
     } else if(role == Qt::DisplayRole) {
         // Display role?
 
@@ -217,33 +245,24 @@ QVariant RemoteModel::data(const QModelIndex& index, int role) const
                 return item->value(RemoteModelColumnName);
             }
         case 1:
-            {
-                if(type == "folder")
-                    return QVariant();
-                return item->value(RemoteModelColumnCommitId);
-            }
-        case 2:
             {
                 return item->value(RemoteModelColumnLastModified);
             }
-        case 3:
+        case 2:
             {
                 // Folders don't have a size
                 if(type == "folder")
                     return QVariant();
 
                 // Convert size to human readable format
-                float size = item->value(RemoteModelColumnSize).toLongLong();
-                QStringList list;
-                list << "KiB" << "MiB" << "GiB" << "TiB";
-                QStringListIterator it(list);
-                QString unit(tr("bytes"));
-                while(size >= 1024.0f && it.hasNext())
-                {
-                    unit = it.next();
-                    size /= 1024.0;
-                }
-                return QString().setNum(size, 'f', 2).remove(".00") + " " + unit;
+                unsigned int size = item->value(RemoteModelColumnSize).toUInt();
+                return humanReadableSize(size);
+            }
+        case 3:
+            {
+                if(type == "folder")
+                    return QVariant();
+                return item->value(RemoteModelColumnCommitId);
             }
         }
     }
@@ -258,7 +277,7 @@ QVariant RemoteModel::headerData(int section, Qt::Orientation orientation, int r
         return QAbstractItemModel::headerData(section, orientation, role);
 
     // Return header string depending on column
-    return headerList.at(section);
+    return headerList.at(static_cast<size_t>(section));
 }
 
 int RemoteModel::rowCount(const QModelIndex& parent) const
@@ -272,7 +291,7 @@ int RemoteModel::rowCount(const QModelIndex& parent) const
 
 int RemoteModel::columnCount(const QModelIndex& /*parent*/) const
 {
-    return headerList.size();
+    return static_cast<int>(headerList.size());
 }
 
 bool RemoteModel::hasChildren(const QModelIndex& parent) const
@@ -306,7 +325,9 @@ void RemoteModel::fetchMore(const QModelIndex& parent)
 
     // Fetch item URL
     item->setFetchedDirectoryList(true);
-    remoteDatabase.fetch(item->value(RemoteModelColumnUrl).toString(), RemoteDatabase::RequestTypeDirectory, currentClientCert, parent);
+    RemoteNetwork::get().fetch(item->value(RemoteModelColumnUrl).toUrl(), RemoteNetwork::RequestTypeCustom, currentClientCert, [this, parent](const QByteArray& reply) {
+        parseDirectoryListing(reply, parent);
+    });
 }
 
 const QString& RemoteModel::currentClientCertificate() const
