@@ -1,5 +1,6 @@
 #include "Settings.h"
 
+#include <QApplication>
 #include <QDir>
 #include <QSettings>
 #include <QColor>
@@ -8,27 +9,33 @@
 #include <QStandardPaths>
 #include <QPalette>
 
-QHash<QString, QVariant> Settings::m_hCache;
+std::unordered_map<std::string, QVariant> Settings::m_hCache;
+int Settings::m_defaultFontSize;
 
-QVariant Settings::getValue(const QString& group, const QString& name)
+static bool ends_with(const std::string& str, const std::string& with)
+{
+    return str.rfind(with) == str.size() - with.size();
+}
+
+QVariant Settings::getValue(const std::string& group, const std::string& name)
 {
     // Have a look in the cache first
     auto cacheIndex = m_hCache.find(group + name);
     if(cacheIndex != m_hCache.end())
     {
-        return cacheIndex.value();
+        return cacheIndex->second;
     } else {
         // Nothing found in the cache, so get the value from the settings file or get the default value if there is no value set yet
         QSettings settings(QApplication::organizationName(), QApplication::organizationName());
-        QVariant value = settings.value(group + "/" + name, getDefaultValue(group, name));
+        QVariant value = settings.value(QString::fromStdString(group + "/" + name), getDefaultValue(group, name));
 
         // Store this value in the cache for further usage and return it afterwards
-        m_hCache.insert(group + name, value);
+        m_hCache.insert({group + name, value});
         return value;
     }
 }
 
-void Settings::setValue(const QString& group, const QString& name, const QVariant& value, bool dont_save_to_disk)
+void Settings::setValue(const std::string& group, const std::string& name, const QVariant& value, bool dont_save_to_disk)
 {
     // Sometime the value has to be saved for the current session only but get discarded when the application exits.
     // In order to achieve this this flag can be set which disables the save to disk mechanism and only leaves the save to cache part active.
@@ -36,8 +43,8 @@ void Settings::setValue(const QString& group, const QString& name, const QVarian
     {
         // Set the group and save the given value
         QSettings settings(QApplication::organizationName(), QApplication::organizationName());
-        settings.beginGroup(group);
-        settings.setValue(name, value);
+        settings.beginGroup(QString::fromStdString(group));
+        settings.setValue(QString::fromStdString(name), value);
         settings.endGroup();
     }
 
@@ -45,7 +52,7 @@ void Settings::setValue(const QString& group, const QString& name, const QVarian
     m_hCache[group + name] = value;
 }
 
-QVariant Settings::getDefaultValue(const QString& group, const QString& name)
+QVariant Settings::getDefaultValue(const std::string& group, const std::string& name)
 {
     // db/defaultencoding?
     if(group == "db" && name == "defaultencoding")
@@ -73,11 +80,15 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
 
     // db/prefetchsize?
     if(group == "db" && name == "prefetchsize")
-        return 50000;
+        return 50000U;
 
     // db/defaultsqltext?
     if(group == "db" && name == "defaultsqltext")
-        return "";
+        return QString();
+
+    // db/fontsize?
+    if(group == "db" && name == "fontsize")
+        return 10;
 
     // exportcsv/firstrowheader?
     if(group == "exportcsv" && name == "firstrowheader")
@@ -131,11 +142,15 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
 
     // MainWindow/geometry?
     if(group == "MainWindow" && name == "geometry")
-        return "";
+        return QString();
 
     // MainWindow/windowState?
     if(group == "MainWindow" && name == "windowState")
-        return "";
+        return QString();
+
+    // MainWindow/openTabs?
+    if(group == "MainWindow" && name == "openTabs")
+        return QString();
 
     // SQLLogDock/Log?
     if(group == "SQLLogDock" && name == "Log")
@@ -149,13 +164,37 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
     if(group == "General" && name == "language")
         return QLocale::system().name();
 
+    // General/appStyle
+    if(group == "General" && name == "appStyle")
+        return static_cast<int>(FollowDesktopStyle);
+
     // General/toolbarStyle
     if(group == "General" && name == "toolbarStyle")
         return static_cast<int>(Qt::ToolButtonTextBesideIcon);
 
+    // General/toolbarStyleStructure
+    if(group == "General" && name == "toolbarStyleStructure")
+        return static_cast<int>(Qt::ToolButtonTextBesideIcon);
+
+    // General/toolbarStyleBrowse
+    if(group == "General" && name == "toolbarStyleBrowse")
+        return static_cast<int>(Qt::ToolButtonIconOnly);
+
+    // General/toolbarStyleSql
+    if(group == "General" && name == "toolbarStyleSql")
+        return static_cast<int>(Qt::ToolButtonIconOnly);
+
+    // General/toolbarStyleEditCell
+    if(group == "General" && name == "toolbarStyleEditCell")
+        return static_cast<int>(Qt::ToolButtonIconOnly);
+
     if(group == "General" && name == "DBFileExtensions")
         return QObject::tr("SQLite database files (*.db *.sqlite *.sqlite3 *.db3)");
 
+    // General/fontsize
+    if(group == "General" && name == "fontsize")
+        return m_defaultFontSize;
+
     // checkversion group?
     if(group == "checkversion")
     {
@@ -178,10 +217,14 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
             return 5000;
         if(name == "complete_threshold")
             return 1000;
+        if(name == "image_preview")
+            return false;
         if(name == "indent_compact")
             return false;
         if(name == "auto_switch_mode")
             return true;
+        if(name == "editor_word_wrap")
+            return true;
         if(name == "null_text")
             return "NULL";
         if(name == "blob_text")
@@ -190,83 +233,28 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
             return "\\";
         if(name == "filter_delay")
             return 200;
-        if(name == "null_fg_colour")
-            return QColor(Qt::lightGray).name();
-        if(name == "null_bg_colour")
-            return QPalette().color(QPalette::Active, QPalette::Base).name();
-        if(name == "reg_fg_colour")
-            return QPalette().color(QPalette::Active, QPalette::Text).name();
-        if(name == "reg_bg_colour")
-            return QPalette().color(QPalette::Active, QPalette::Base).name();
-        if(name == "bin_fg_colour")
-            return QColor(Qt::lightGray).name();
-        if(name == "bin_bg_colour")
-            return QPalette().color(QPalette::Active, QPalette::Base).name();
+        if(ends_with(name, "colour"))
+            return getDefaultColorValue(group, name, FollowDesktopStyle);
     }
 
     // syntaxhighlighter?
     if(group == "syntaxhighlighter")
     {
         // Bold? Only tables, functions and keywords are bold by default
-        if(name.right(4) == "bold")
+        if(ends_with(name, "bold"))
             return name == "keyword_bold" || name == "table_bold" || name == "function_bold";
 
         // Italic? Nothing by default
-        if(name.right(6) == "italic")
+        if(ends_with(name, "italic"))
             return false;
 
         // Underline? Nothing by default
-        if(name.right(9) == "underline")
+        if(ends_with(name, "underline"))
             return false;
 
         // Colour?
-        if(name.right(6) == "colour")
-        {
-            QColor backgroundColour = QPalette().color(QPalette::Active, QPalette::Base);
-            QColor foregroundColour = QPalette().color(QPalette::Active, QPalette::Text);
-
-            if(name == "foreground_colour")
-                return foregroundColour.name();
-            else if(name == "background_colour")
-                return backgroundColour.name();
-
-            // Detect and provide sensible defaults for dark themes
-            if (backgroundColour.value() < foregroundColour.value()) {
-                if(name == "keyword_colour")
-                  return QColor(82, 148, 226).name();
-                else if(name == "function_colour")
-                    return QColor(Qt::yellow).name();
-                else if(name == "table_colour")
-                    return QColor(Qt::cyan).name();
-                else if(name == "comment_colour")
-                    return QColor(Qt::green).name();
-                else if(name == "identifier_colour")
-                    return QColor(Qt::magenta).name();
-                else if(name == "string_colour")
-                    return QColor(Qt::lightGray).name();
-                else if(name == "currentline_colour")
-                    return backgroundColour.lighter(150).name();
-                else if(name == "background_colour")
-                    return backgroundColour.name();
-            } else {
-                if(name == "keyword_colour")
-                    return QColor(Qt::darkBlue).name();
-                else if(name == "function_colour")
-                    return QColor(Qt::blue).name();
-                else if(name == "table_colour")
-                    return QColor(Qt::darkCyan).name();
-                else if(name == "comment_colour")
-                    return QColor(Qt::darkGreen).name();
-                else if(name == "identifier_colour")
-                    return QColor(Qt::darkMagenta).name();
-                else if(name == "string_colour")
-                    return QColor(Qt::red).name();
-                else if(name == "currentline_colour")
-                    return QColor(236, 236, 245).name();
-                else if(name == "background_colour")
-                    return backgroundColour.name();
-            }
-        }
+        if(ends_with(name, "colour"))
+            return getDefaultColorValue(group, name, FollowDesktopStyle);
     }
 
     // editor/font?
@@ -318,6 +306,18 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
     if(group == "editor" && name == "horizontal_tiling")
         return false;
 
+    // editor/splitter1_sizes?
+    if(group == "editor" && name == "splitter1_sizes")
+        return QVariant();
+
+    // editor/splitter2_sizes?
+    if(group == "editor" && name == "splitter2_sizes")
+        return QVariant();
+
+    // editor/close_button_on_tabs?
+    if(group == "editor" && name == "close_button_on_tabs")
+        return true;
+
     // extensions/list?
     if(group == "extensions" && name == "list")
         return QStringList();
@@ -369,10 +369,117 @@ QVariant Settings::getDefaultValue(const QString& group, const QString& name)
 #endif
     }
 
+    // Proxy settings
+    if(group == "proxy")
+    {
+        // Use system settings by default
+        if(name == "type")
+            return "system";
+    }
+
     // Unknown combination of group and name? Return an invalid QVariant!
     return QVariant();
 }
 
+QColor Settings::getDefaultColorValue(const std::string& group, const std::string& name, AppStyle style)
+{
+    // Data Browser/NULL & Binary Fields
+    if(group == "databrowser")
+    {
+        switch (style) {
+        case FollowDesktopStyle :
+            if(name == "null_fg_colour")
+                return QColor(Qt::lightGray).name();
+            if(name == "null_bg_colour")
+                return QPalette().color(QPalette::Active, QPalette::Base).name();
+            if(name == "reg_fg_colour")
+                return QPalette().color(QPalette::Active, QPalette::Text).name();
+            if(name == "reg_bg_colour")
+                return QPalette().color(QPalette::Active, QPalette::Base).name();
+            if(name == "bin_fg_colour")
+                return QColor(Qt::lightGray).name();
+            if(name == "bin_bg_colour")
+                return QPalette().color(QPalette::Active, QPalette::Base).name();
+            break;
+        case DarkStyle :
+            if(name == "null_fg_colour")
+                return QColor("#787878");
+            if(name == "null_bg_colour")
+                return QColor("#19232D");
+            if(name == "reg_fg_colour")
+                return QColor("#F0F0F0");
+            if(name == "reg_bg_colour")
+                return QColor("#19232D");
+            if(name == "bin_fg_colour")
+                return QColor("#787878");
+            if(name == "bin_bg_colour")
+                return QColor("#19232D");
+            break;
+        }
+    }
+
+    // syntaxhighlighter?
+    if(group == "syntaxhighlighter")
+    {
+        // Colour?
+        if(ends_with(name, "colour"))
+        {
+            QColor backgroundColour;
+            QColor foregroundColour;
+            switch (style) {
+            case FollowDesktopStyle :
+                backgroundColour = QPalette().color(QPalette::Active, QPalette::Base);
+                foregroundColour = QPalette().color(QPalette::Active, QPalette::Text);
+                break;
+            case DarkStyle :
+                foregroundColour = QColor("#F0F0F0");
+                backgroundColour = QColor("#19232D");
+                break;
+            }
+            if(name == "foreground_colour")
+                return foregroundColour;
+            else if(name == "background_colour")
+                return backgroundColour;
+
+            // Detect and provide sensible defaults for dark themes
+            if (backgroundColour.value() < foregroundColour.value()) {
+                if(name == "keyword_colour")
+                    return QColor(82, 148, 226);
+                else if(name == "function_colour")
+                    return QColor(Qt::yellow);
+                else if(name == "table_colour")
+                    return QColor(Qt::cyan);
+                else if(name == "comment_colour")
+                    return QColor(Qt::green);
+                else if(name == "identifier_colour")
+                    return QColor(Qt::magenta);
+                else if(name == "string_colour")
+                    return QColor(Qt::lightGray);
+                else if(name == "currentline_colour")
+                    return backgroundColour.lighter(150);
+            } else {
+                if(name == "keyword_colour")
+                    return QColor(Qt::darkBlue);
+                else if(name == "function_colour")
+                    return QColor(Qt::blue);
+                else if(name == "table_colour")
+                    return QColor(Qt::darkCyan);
+                else if(name == "comment_colour")
+                    return QColor(Qt::darkGreen);
+                else if(name == "identifier_colour")
+                    return QColor(Qt::darkMagenta);
+                else if(name == "string_colour")
+                    return QColor(Qt::red);
+                else if(name == "currentline_colour")
+                    return QColor(236, 236, 245);
+            }
+        }
+    }
+
+    // Unknown combination of group and name? Return an invalid QColor!
+    return QColor();
+}
+
 void Settings::restoreDefaults ()
 {
     QSettings settings(QApplication::organizationName(), QApplication::organizationName());
