@@ -1,13 +1,16 @@
-#ifndef SQLEXECUTIONAREA_H
+﻿#ifndef SQLEXECUTIONAREA_H
 #define SQLEXECUTIONAREA_H
 
 #include <QWidget>
+#include <QFileSystemWatcher>
 
 class SqlTextEdit;
 class SqliteTableModel;
 class DBBrowserDB;
 class ExtendedTableWidget;
 
+class QTextEdit;
+
 namespace Ui {
 class SqlExecutionArea;
 }
@@ -18,10 +21,14 @@ class SqlExecutionArea : public QWidget
 
 public:
     explicit SqlExecutionArea(DBBrowserDB& _db, QWidget* parent = nullptr);
-    ~SqlExecutionArea();
+    ~SqlExecutionArea() override;
 
     QString getSql() const;
     QString getSelectedSql() const;
+    void setSql(const QString& sql);
+
+    void openFile(const QString& filename);
+    void saveFile(const QString& filename);
 
     QString fileName() const { return sqlFileName; }
     void setFileName(const QString& filename) { sqlFileName = filename; }
@@ -29,11 +36,17 @@ public:
     SqliteTableModel* getModel() { return model; }
     SqlTextEdit* getEditor();
     ExtendedTableWidget *getTableResult();
+    QTextEdit* getStatusEdit();
+
+    bool inErrorState() const { return error_state; }
+
+    // Save window state to settings
+    static void saveState();
 
 public slots:
-    virtual void finishExecution(const QString& result, const bool ok);
-    virtual void saveAsCsv();
-    virtual void reloadSettings();
+    void finishExecution(const QString& result, const bool ok);
+    void saveAsCsv();
+    void reloadSettings();
     void fetchedData();
     void setFindFrameVisibility(bool show);
 
@@ -43,6 +56,8 @@ private slots:
     void findLineEdit_textChanged(const QString& text);
     void hideFindFrame();
 
+    void fileChanged(const QString& filename);
+
 signals:
     void findFrameVisibilityChanged(bool visible);
 
@@ -51,9 +66,11 @@ private:
     DBBrowserDB& db;
     SqliteTableModel* model;
     QString sqlFileName;
+    QFileSystemWatcher fileSystemWatch;
     Ui::SqlExecutionArea* ui;
     bool m_columnsResized;              // This is set to true if the columns of the table view were already adjusted to fit their contents
     bool showErrorIndicators;
+    bool error_state;
 };
 
 #endif
