@@ -3,7 +3,6 @@
 
 #include "ExtendedScintilla.h"
 
-class FindReplaceDialog;
 class SqlUiLexer;
 
 /**
@@ -16,12 +15,12 @@ class SqlTextEdit : public ExtendedScintilla
 
 public:
     explicit SqlTextEdit(QWidget *parent = nullptr);
-    virtual ~SqlTextEdit();
 
     static SqlUiLexer* sqlLexer;
 
 public slots:
     void reloadSettings();
+    void toggleBlockComment();
 
 };
 
